/**
 * prefUtils.js
 * 0.2
 * 2004-05-17
 * Daniel Lindkvist
 **/

///////////////////////////
// Used With explicit permission (and is GPL) from Daniel Lindkvist
// slightly modified by avih (avihpit [at] yahoo [dot] com) for smoothwheel (2004-11-07)
//   (added onReset handler to load defaults)
///////////////////////////

// settings
var _DEBUG = false;
var AUTO_SET_PREFS = true;

if(_DEBUG) dump("in prefutils\n");


var prefutils =
{
  _onLoad: function(aDialogId, doResetValues)
  {
    var aWindow = document.getElementById(aDialogId);
    if(_DEBUG) dump("in onLoad\n");
    if(!aWindow) {
      if(_DEBUG) dump("No dialog id supplied?\n");
      return;
    }
    var prefElements = aWindow.getElementsByAttribute("prefstring", "*");
    if(!prefElements) {
      if(_DEBUG) dump("prefElements empty\n");
      return;
    }
    for(var i=0; i<prefElements.length; i++) {
      var prefstring = prefElements[i].getAttribute("prefstring");
      var prefid = prefElements[i].getAttribute("id");
      var preftype = prefElements[i].getAttribute("preftype");
      var elt = prefElements[i].localName;
      var prefdefval = prefElements[i].getAttribute("prefdefval");
      if(!prefdefval)
        prefdefval = "";

      if (!preftype) {
        if (elt == "textbox")
          preftype = "string";
        else if (elt == "checkbox" || elt == "button")
          preftype = "bool";
        else if (elt == "radiogroup" || elt == "menulist")
          preftype = "int";
      }
      var prefvalue = prefutils.getPref(preftype, prefstring, prefdefval);
      if (doResetValues){
        prefvalue="!/!ERROR_UNDEFINED_PREF!/!";
      } else {
      }
      if(_DEBUG) dump("pref collected: "+ prefvalue+"\n");
      if(_DEBUG) dump("elementtype: "+ elt+"\n\n");
      if(prefvalue == "!/!ERROR_UNDEFINED_PREF!/!") {
        prefvalue = prefdefval;
        if(AUTO_SET_PREFS)
          prefutils.setPref(preftype, prefstring, prefvalue);
      }
      switch(elt) {
        case "textbox":
          prefElements[i].value = prefvalue;
          break;
        case "checkbox":
          prefElements[i].checked = prefvalue;
          break;
        case "button":
          prefElements[i].checked = prefvalue;
          break;
        case "radiogroup":
          prefElements[i].value = prefvalue;
          break;
        case "menulist":
          prefElements[i].selectedIndex = prefvalue;
          break;
      }
    }
    if(_DEBUG) dump("init finished\n");
  },// end init

  onLoad: function(aDialogId)
  {
    return this._onLoad(aDialogId, false);
  },
  
  onReset: function(aDialogId){
    return this._onLoad(aDialogId, true);
  },//end onReset

  onOK: function(aDialogId)
  {
    var aWindow = document.getElementById(aDialogId);
    var prefElements = aWindow.getElementsByAttribute("prefstring", "*");

    for(var i=0; i<prefElements.length; i++) {
      var prefstring = prefElements[i].getAttribute("prefstring");
      var preftype = prefElements[i].getAttribute("preftype");

      var elt = prefElements[i].localName;
      if (!preftype) {
        if (elt == "textbox")
          preftype = "string";
        else if (elt == "checkbox" || elt == "button")
          preftype = "bool";
        else if (elt == "radiogroup" || elt == "menulist")
          preftype = "int";
      }
      var prefvalue; 
      if (elt == "checkbox")
        prefvalue = prefElements[i].checked;
      else if (elt == "button")
        prefvalue = prefElements[i].checked;
      else if (elt == "radiogroup" || elt == "menulist")
        prefvalue = prefElements[i].selectedIndex;
      else if (elt == "textbox")
        prefvalue = prefElements[i].value;

      prefutils.setPref(preftype, prefstring, prefvalue);
      if(_DEBUG) dump("setpref:" +prefstring+" "+ prefvalue + "\n");
    }
    return true;
  },// end onOk


  getPref: function(aPrefType, aPrefString, aDefaultFlag)
  {
    if(_DEBUG) dump("in getpref\n");
    var pref = Components.classes["@mozilla.org/preferences-service;1"]
        .getService(Components.interfaces.nsIPrefBranch);
    try {
      switch(aPrefType) {
        case "bool":
          return pref.getBoolPref(aPrefString);
          //return !aDefaultFlag ? pref.getBoolPref(aPrefString) : pref.getDefaultBoolPref(aPrefString);
        case "int":
          return pref.getIntPref(aPrefString);
          //return !aDefaultFlag ? pref.getIntPref(aPrefString) : pref.getDefaultIntPref(aPrefString);
        case "localizedstring":
          return pref.getLocalizedUnicharPref(aPrefString); // not working?
        case "color":
        case "string":
        default:
          return pref.getCharPref(aPrefString); 
          //return !aDefaultFlag ? pref.copyUnicharPref(aPrefString) : pref.copyDefaultUnicharPref(aPrefString);
      }
    }catch(e) {
      if(_DEBUG) {
        dump("*** no default pref for " + aPrefType + " pref: " + aPrefString + "\n");
        dump(e + "\n");
      }
    }
    return "!/!ERROR_UNDEFINED_PREF!/!";
  },


  setPref: function(aPrefType, aPrefString, aValue)
  {
    var pref = Components.classes["@mozilla.org/preferences-service;1"]
        .getService(Components.interfaces.nsIPrefBranch);
    try {
      switch (aPrefType) {
        case "bool":
          pref.setBoolPref(aPrefString, aValue);
          break;
        case "int":
          pref.setIntPref(aPrefString, aValue);
          break;
        case "color":
        case "string":
        case "localizedstring":
        default:
          pref.setCharPref(aPrefString, aValue);
          break;
      }
    }catch(e){
      if(_DEBUG) dump(e + "\n");
    }
  }
};// end prefutils